<?php
namespace App\Http;


use Illuminate\Foundation\Http\Kernel as HttpKernel;

class Kernel extends HttpKernel
{
    /**
     * The application's global HTTP middleware stack.
     *
     * These middleware are run during every request to your application.
     *
     * @var array
     */
    protected $middleware = [
        \Illuminate\Foundation\Http\Middleware\CheckForMaintenanceMode::class,
    ];

    /**
     * The application's route middleware groups.
     *
     * @var array
     */
    protected $middlewareGroups = [
        'web' => [
            \App\Http\Middleware\EncryptCookies::class,
            \Illuminate\Cookie\Middleware\AddQueuedCookiesToResponse::class,
            \Illuminate\Session\Middleware\StartSession::class,
            \Illuminate\View\Middleware\ShareErrorsFromSession::class,
            \App\Http\Middleware\VerifyCsrfToken::class,
            //\Illuminate\Routing\Middleware\SubstituteBindings::class,
            \Illuminate\Foundation\Http\Middleware\TrimStrings::class,
            \App\Http\Middleware\DuplicateSubmissionCheck::class,
            \App\Http\Middleware\QueryLogging::class,
            \App\Http\Middleware\StartupCheck::class,
        ],
        'api' => [
            \App\Http\Middleware\QueryLogging::class,
            \App\Http\Middleware\ApiCheck::class,
        ],
        /*
        'api' => [
            'throttle:60,1',
            'bindings',
        ],
        */
    ];

    /**
     * The application's route middleware.
     *
     * These middleware may be assigned to groups or used individually.
     *
     * @var array
     */
    protected $routeMiddleware = [
        'auth' => \App\Http\Middleware\Authenticate::class,
        'auth.basic' => \Illuminate\Auth\Middleware\AuthenticateWithBasicAuth::class,
        'bindings' => \Illuminate\Routing\Middleware\SubstituteBindings::class,
        'can' => \Illuminate\Auth\Middleware\Authorize::class,
        'guest' => \App\Http\Middleware\RedirectIfAuthenticated::class,
        'throttle' => \Illuminate\Routing\Middleware\ThrottleRequests::class,
        'lookup' => \App\Http\Middleware\DatabaseLookup::class,
        'permissions.required' => \App\Http\Middleware\PermissionsRequired::class,
        'migration' => \App\Http\Middleware\EligibleForMigration::class,
        'migration_channel' => \App\Http\Middleware\MigrationLookup::class,
    ];
}
